@@ -15,11 +15,10 @@
 #define _LINUX_NETDEVICE_H /* very crude hack for glibc2 */
 #include <linux/if_arp.h>
 #include <linux/if_ether.h>
-#include <linux/atm.h>
+#include <atm.h>
 #include <linux/atmarp.h>
 #include <linux/atmclip.h>
 
-#include "atm.h"
 #include "atmd.h"
 #include "atmarp.h"
 #include "atmarpd.h"
