@@ -15,7 +15,7 @@
 #include <netdb.h>
 #include <net/if_arp.h>
 
-#include <linux/atm.h>
+#include <stdint.h>
 #include <linux/atmarp.h>
 #include <linux/atmclip.h>
 
