@@ -15,13 +15,12 @@
 #include <sys/ioctl.h>
 #include <net/if.h>
 #include <netinet/in.h>
-#include <linux/atm.h>
+#include <atm.h>
 #include <linux/atmclip.h> /* for CLIP_DEFAULT_IDLETIMER */
 #include <linux/atmarp.h>
 #define _LINUX_NETDEVICE_H /* glibc2 */
 #include <linux/if_arp.h>
 
-#include "atm.h"
 #include "atmd.h"
 #include "atmarpd.h"
 
