@@ -8,7 +8,7 @@
 
 #include <stdint.h>
 #include <sys/socket.h> /* for struct sockaddr */
-#include <linux/atm.h> /* for struct sockaddr_atmsvc */
+#include <atm.h> /* for struct sockaddr_atmsvc */
 
 
 void open_all(void);
