@@ -27,6 +27,24 @@
 
 
 static int s_tcp,s_krn;
+static int debug = 0;
+
+
+static void net_write(void *data,int size)
+{
+    int wrote;
+
+    wrote = write(s_tcp,data,size);
+    if (wrote < 0) {
+	perror("write to TCP");
+	exit(1);
+    }
+    if (!wrote) exit(0); /* EOF */
+    if (wrote != size) {
+	fprintf(stderr,"bad write (%d != %d)\n",wrote,size);
+	exit(1);
+    }
+}
 
 
 /*
@@ -38,11 +56,12 @@ static int s_tcp,s_krn;
 static void from_tcp(void)
 {
     static char buf[MAX_PACKET];
+    static struct atmtcp_hdr *hdr = (struct atmtcp_hdr *) buf;
     static int bytes = 0;
     int ret,size;
 
     size = sizeof(struct atmtcp_hdr)-bytes;
-    if (size <= 0) size += ntohl(((struct atmtcp_hdr *) buf)->length);
+    if (size <= 0) size += ntohl(hdr->length);
     ret = read(s_tcp,buf+bytes,size);
     if (ret < 0) {
 	perror("read from TCP");
@@ -53,11 +72,13 @@ static void from_tcp(void)
     if (bytes < sizeof(struct atmtcp_hdr)) return;
     if (ntohl(((struct atmtcp_hdr *) buf)->length) > ATM_MAX_AAL5_PDU) {
 	fprintf(stderr,"giant PDU (length = %d) received\n",
-	  (unsigned int) ntohl(((struct atmtcp_hdr *) buf)->length));
+	  (unsigned int) ntohl(hdr->length));
 	exit(1);
     }
-    if (bytes < sizeof(struct atmtcp_hdr)+
-      ntohl(((struct atmtcp_hdr *) buf)->length)) return;
+    if (bytes < sizeof(struct atmtcp_hdr)+ntohl(hdr->length)) return;
+    if (debug)
+	fprintf(stderr,"TCP %d.%d, %d bytes\n",ntohs(hdr->vpi),ntohs(hdr->vci),
+	  (unsigned int) ntohl(hdr->length));
     size = write(s_krn,buf,bytes);
     if (size < 0) perror("write to kernel");
     else if (size != bytes) {
@@ -71,7 +92,8 @@ static void from_tcp(void)
 static void to_tcp(void)
 {
     char buf[MAX_PACKET];
-    int ret,wrote;
+    struct atmtcp_hdr *hdr = (struct atmtcp_hdr *) buf;
+    int ret;
 
     ret = read(s_krn,buf,MAX_PACKET);
     if (ret < 0) {
@@ -83,69 +105,88 @@ static void to_tcp(void)
 	fprintf(stderr,"kernel message too small (%d)\n",ret);
 	exit(1);
     }
-    if (ret != sizeof(struct atmtcp_hdr)+
-      ntohl(((struct atmtcp_hdr *) buf)->length)) {
+    if (debug)
+	fprintf(stderr,"KRN %d.%d, %d bytes\n",ntohs(hdr->vpi),ntohs(hdr->vci),
+	  (unsigned int) ntohl(hdr->length));
+    if (ret != sizeof(struct atmtcp_hdr)+ntohl(hdr->length)) {
 	fprintf(stderr,"invalid kernel message\n");
 	exit(1);
     }
-    wrote = write(s_tcp,buf,ret);
-    if (wrote < 0) {
-	perror("write to TCP");
-	exit(1);
-    }
-    if (!wrote) exit(0); /* EOF */
-    if (wrote != ret) {
-	fprintf(stderr,"bad write (%d != %d)\n",wrote,ret);
-	exit(1);
-    }
+    net_write(buf,ret);
 }
 
 
 static void usage(const char *name)
 {
-    fprintf(stderr,"%s [ -b ] [ -i itf ] -l [ <lclport> ]\n",name);
-    fprintf(stderr,"%s [ -b ] [ -i itf ] -c <host> [ <rmtport> ]\n",name);
+    fprintf(stderr,"%s [ -b ] [ -d ] [ -i itf ] -l [ <lclport> ]\n",name);
+    fprintf(stderr,"%s [ -b ] [ -d ] [ -i itf ] -c <host> [ <rmtport> ]\n",
+      name);
+    fprintf(stderr,"%s [ -b ] [ -d ] [ -i itf ] -s <host> <line> "
+      "[ <rmtport> ]\n",name);
     exit(1);
 }
 
 
 int main(int argc,char **argv)
 {
+    enum { m_undefined,m_listen,m_connect,m_switch } mode;
     struct sockaddr_in addr;
     const char *name;
-    int background,listen_mode,port,itf;
+    char *end;
+    char *host,*line;
+    int background,port,itf;
     int s2;
+    int c;
 
     name = argv[0];
+    mode = m_undefined;
     background = 0;
+    host = line = NULL; /* for gcc */
     itf = -1;
-    while (argc > 2) {
-	if (!strcmp(argv[1],"-b")) {
-	    if (background) usage(name);
-	    background = 1;
-	    argc--;
-	    argv++;
+    while ((c = getopt(argc,argv,"bdi:lc:s:")) != EOF)
+	switch (c) {
+	    case 'b':
+		background = 1;
+		break;
+	    case 'd':
+		debug = 1;
+		break;
+	    case 'i':
+		if (itf != -1) usage(name);
+		itf = strtoul(optarg,&end,10);
+		if (*end) usage(name);
+		break;
+	    case 'l':
+		if (mode != m_undefined) usage(name);
+		mode = m_listen;
+		break;
+	    case 'c':
+		if (mode != m_undefined) usage(name);
+		mode = m_connect;
+		host = optarg;
+		break;
+	    case 's':
+		if (mode != m_undefined) usage(name);
+		mode = m_switch;
+		host = optarg;
+		break;
+	    default:
+		usage(name);
         }
-	else if (!strcmp(argv[1],"-i")) {
-	    char *end;
-
-	    if (itf != -1) usage(name);
-	    itf = strtoul(argv[2],&end,10);
-	    if (*end) usage(name);
-	    argc -= 2;
-	    argv += 2;
-        }
-	else break;
-    }
-    if (argc < 2) usage(name);
-    listen_mode = !strcmp(argv[1],"-l");
-    if (listen_mode) {
-	if (argc > 3) usage(name);
-	port = argc == 3 ? atoi(argv[2]) : -1;
-    }
-    else {
-	if (argc > 4 || strcmp(argv[1],"-c")) usage(name);
-	port = argc == 4 ? atoi(argv[3]) : -1;
+    if (mode == m_undefined) usage(name);
+    switch (mode) {
+	case m_switch:
+	    if (argc == optind) usage(name);
+	    line = argv[optind++];
+	    /* fall through */
+	case m_connect:
+	    /* fall through */
+	case m_listen:
+	    if (argc > optind+1) usage(name);
+	    port = argc == optind+1 ? atoi(argv[optind]) : -1;
+	    break;
+	default:
+	    abort();
     }
     if ((s_tcp = socket(PF_INET,SOCK_STREAM,0)) < 0) {
 	perror("socket");
@@ -156,7 +197,7 @@ int main(int argc,char **argv)
     addr.sin_port = htons(port == -1 ?  PORT :  port);
     /*
      * Create atmtcp interface before setting up the TCP connection in order
-     * to make assignment of interface numbers a bit more predictable.
+     * to make the assignment of interface numbers a bit more predictable.
      */
     if ((s_krn = socket(PF_ATMSVC,SOCK_DGRAM,0)) < 0) {
 	perror("socket");
@@ -166,7 +207,7 @@ int main(int argc,char **argv)
 	perror("ioctl SIOCSIFATMTCP");
 	return 1;
     }
-    if (listen_mode) {
+    if (mode == m_listen) {
 	addr.sin_addr.s_addr = htonl(INADDR_ANY);
 	if (bind(s_tcp,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
 	    perror("bind");
@@ -187,11 +228,11 @@ int main(int argc,char **argv)
     else {
 	struct hostent *h;
 
-	h = gethostbyname(argv[2]);
+	h = gethostbyname(host);
 	if (h)
 	    memcpy(&addr.sin_addr.s_addr,h->h_addr,
 	      sizeof(addr.sin_addr.s_addr));
-	else if ((addr.sin_addr.s_addr = inet_addr(argv[2])) == -1) {
+	else if ((addr.sin_addr.s_addr = inet_addr(host)) == -1) {
 		herror(argv[2]);
 		return 1;
 	    }
@@ -199,6 +240,7 @@ int main(int argc,char **argv)
 	    perror("connect");
 	    return 1;
 	}
+	if (mode == m_switch) net_write(line,strlen(line)+1);
     }
     printf("Connected, interface %d\n",itf);
     if (background) {
