@@ -29,14 +29,16 @@
 extern int yyparse(void);
 extern FILE *yyin;
 
-int net = 0;
-int allocate_ci = 1;
 int debug = 0;
 int pretty = A2T_PRETTY | A2T_NAME | A2T_LOCAL;
 int sig_pcr = -1; /* obsolete */
 const char *sig_qos = NULL;
 const char *dump_dir = NULL;
 
+SIGNALING_MODE mode = sm_user;
+int net = 0;
+int allocate_ci = 1;
+
 
 /* ------------------------------ SAAL relays ------------------------------ */
 
@@ -282,8 +284,8 @@ static void usage(const char *name)
 {
     fprintf(stderr,"usage: %s [ -b ] [ -c config_file ] [ -d ] "
       "[ -D dump_dir ]\n"
-      "  [ -l logfile ] [ -n ] [ -N [ -A ] ]  [ -q qos ] [ -t trace_length ]\n"
-      "  [ [itf.]vpi.vci [ socket_path ] ]\n",name);
+      "  [ -l logfile ] [ -n ] [ -m user|network|switch ]  [ -q qos ]\n"
+      "  [ -t trace_length ] [ [itf.]vpi.vci [ socket_path ] ]\n",name);
     exit(1);
 }
 
@@ -301,7 +303,7 @@ int main(int argc,char **argv)
     memset(&signaling_pvc,0,sizeof(signaling_pvc));
     signaling_pvc.sap_addr.vci = 5;
     /* 1st pass to get the -c option */
-    while ((c = getopt(argc,argv,"Abc:dD:l:nNP:q:t:")) != EOF)
+    while ((c = getopt(argc,argv,"Abc:dD:l:m:nNP:q:t:")) != EOF)
 	if (c == 'c') config_file = optarg;
     if (!(yyin = fopen(config_file,"r")))
 	diag(COMPONENT,DIAG_WARN,"%s not found. - Using defaults.",config_file);
@@ -309,7 +311,7 @@ int main(int argc,char **argv)
 	    diag(COMPONENT,DIAG_FATAL,"Error in config file. - Aborting.");
     /* process all other options but -c */
     optind = 0;
-    while ((c = getopt(argc,argv,"Abc:dD:l:nNP:q:t:")) != EOF)
+    while ((c = getopt(argc,argv,"Abc:dD:l:m:nNP:q:t:")) != EOF)
 	switch (c) {
 	    case 'A':
 		allocate_ci = 0;
@@ -334,6 +336,12 @@ int main(int argc,char **argv)
 	    case 'l':
 		set_logfile(optarg);
 		break;
+	    case 'm':
+		if (!strcmp(optarg,"user")) mode = sm_user;
+		else if (!strcmp(optarg,"network")) mode = sm_net;
+		else if (!strcmp(optarg,"switch")) mode = sm_switch;
+		else usage(argv[0]);
+		break;
 	    case 'n':
 		pretty = A2T_PRETTY;
 		break;
@@ -356,7 +364,38 @@ int main(int argc,char **argv)
 	    default:
 		usage(argv[0]);
 	}
-    if (!allocate_ci && !net) usage(argv[0]);
+    switch (mode) {
+	case sm_unknown:
+	    if (net) {
+		if (allocate_ci) {
+		    mode = sm_net;
+		    diag(COMPONENT,DIAG_WARN,"option -N is obsolete, "
+		      "please use  -m network  instead");
+		}
+		else {
+		    mode = sm_switch;
+		    diag(COMPONENT,DIAG_WARN,"options -N -A are obsolete, "
+		      "please use  -m switch  instead");
+		}
+	    }
+	    else if (allocate_ci) mode = sm_user;
+		else usage(argv[0]);
+	    break;
+	case sm_user:
+	    allocate_ci = 1; /* should be ignored anyway */
+	    net = 0;
+	    break;
+	case sm_net:
+	    allocate_ci = 1;
+	    net = 1;
+	    break;
+	case sm_switch:
+	    allocate_ci = 0;
+	    net = 1;
+	    break;
+	default:
+	    abort();
+    }
     if (optind < argc) {
 	if (text2atm(argv[optind],(struct sockaddr *) &signaling_pvc,
 	  sizeof(signaling_pvc),T2A_PVC) < 0)
@@ -388,10 +427,12 @@ int main(int argc,char **argv)
     if (dump_dir)
 	if (chdir(dump_dir) < 0)
 	    diag(COMPONENT,DIAG_ERROR,"chdir %s: %s",dump_dir,strerror(errno));
-    diag(COMPONENT,DIAG_INFO,"Acting as %s side",net ? "NETWORK" : "USER");
+    diag(COMPONENT,DIAG_INFO,"Acting as %s",
+      mode == sm_user ? "USER side" :
+      mode == sm_net ? "NETWORK side" : "SWITCH");
     if (open_all()) return 1;
     init_current_time();
-    init_addr();
+    if (mode != sm_switch) init_addr();
     q_start();
     start_saal(&saal,&ops,NULL);
     saal_estab_req(&saal,NULL,0);
