@@ -33,6 +33,7 @@
 struct timeval now;
 struct sockaddr_atmpvc signaling_pvc;
 
+int stop = 0;
 
 static int kernel = -1,signaling;
 static int need_connect = 0; /* non-zero if connection to kernel isn't
@@ -282,7 +283,7 @@ void poll_loop(void)
     FD_SET(signaling,&perm);
     fds = kernel > signaling ? kernel+1 : signaling+1;
     gettimeofday(&now,NULL);
-    while (1) {
+    while (!stop) {
 	set = perm;
 	poll_signals();
 	/*
