@@ -538,6 +538,17 @@ void from_kernel(struct atmsvc_msg *msg,int size)
 	itf_load(msg->pvc.sap_addr.itf);
 	return;
     }
+    if (msg->type == as_terminate) {
+	if (mode != sm_switch) {
+	    diag(COMPONENT,DIAG_ERROR,"Ignoring as_terminate received in %s "
+	      "mode",mode == sm_user ? "USER" : "NETWORK");
+	    return;
+	}
+	clear_all_calls();
+	stop = 1;
+	diag(COMPONENT,DIAG_INFO,"Going down on as_terminate");
+	return;
+    }
     if (msg->listen_vcc && (msg->type == as_accept || msg->type == as_reject ||
       msg->type == as_identify)) {
 	dispatcher = dispatch_listen;
