@@ -64,6 +64,7 @@ typedef struct _socket {
  * stable enough to be useful to test the "real" thing against it.
  */
 
+typedef enum { sm_unknown,sm_user,sm_net,sm_switch } SIGNALING_MODE;
 
 extern const CALL_STATE state_map[];
 extern const PARTY_STATE eps_map[];
@@ -74,13 +75,16 @@ extern unsigned char q_buffer[];
 
 #define DEFAULT_TRACE_SIZE 20
 
-extern int net;
-extern int allocate_ci;
+extern SIGNALING_MODE mode;
+extern int net; /* @@@ remove soon */
+extern int allocate_ci; /* @@@ remove soon */
 extern int pretty;
 extern int sig_pcr;
 extern const char *sig_qos;
 extern const char *dump_dir;
 
+extern int stop;
+
 
 #define SEND_ERROR(vcc,code) \
   send_kernel(vcc,0L,as_error,code,NULL,NULL,NULL,NULL,NULL)
@@ -100,6 +104,7 @@ void from_net(void *msg,int size);
 void to_signaling(void *msg,int size);
 void saal_failure(void);
 void saal_okay(void);
+void clear_all_calls(void);
 void clear_all_calls_on_T309(void);
 
 SOCKET *new_sock(unsigned long id);
