@@ -600,7 +600,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 }
 
 
-static void clear_all_calls(void)
+void clear_all_calls(void)
 {
     SOCKET *curr,*next;
 
