@@ -25,6 +25,11 @@ typedef struct _fab {
 static CALL *calls;
 
 
+void fab_start(void (*port_notify)(int number,int up))
+{
+}
+
+
 void fab_init(CALL *call)
 {
     PRV(call) = alloc_t(FAB);
@@ -47,6 +52,12 @@ void fab_destroy(CALL *call)
 }
 
 
+/*
+ * This function is rather simple-minded, because it only considers a single
+ * port. Should go directly to the fabric control. @@@
+ */
+
+
 static int vci_exists(int vci,int threshold)
 {
     CALL *call;
@@ -89,8 +100,14 @@ void fab_op(CALL *call,int op,const struct atm_qos *qos,
       op & _RM_SHIFT(RM_PATH_TX) ? " PATH_TX" : "",
       op & _RM_SHIFT(RM_PATH_RX) ? " PATH_RX" : "");
     if (op & (RM_RSV(RM_IN) | RM_CLAIM(RM_IN)))
-	if (!check_ci(&call->in.pvc)) callback(call,0,user);
+	if (!check_ci(&call->in.pvc)) {
+	    callback(call,0,user);
+	    return;
+	}
     if (op & (RM_RSV(RM_OUT) | RM_CLAIM(RM_OUT)))
-	if (!check_ci(&call->out.pvc)) callback(call,0,user);
+	if (!check_ci(&call->out.pvc)) {
+	    callback(call,0,user);
+	    return;
+	}
     callback(call,1,user);
 }
