@@ -29,6 +29,15 @@
 
 /* --- Provided by fabric control ------------------------------------------ */
 
+/*
+ * Initialize the fabric interface. port_notify is invoked whenever a port
+ * is added to or removed from the switch. fab_start may invoke port_notify
+ * before returning. port_notify(X,0) is not invoked until all fab_ops on
+ * that port have completed.
+ */
+
+void fab_start(void (*port_notify)(int number,int up));
+
 /*
  * Initialize the fabric-specific part of a call structure, i.e. allocate a
  * fab-specific descriptor and attach it to call->fab. This function is called
